var vows = require('vows')
var assert = require('assert')
var fire = require('fire')
var fireAssert = require('../index.js')
var path = require('path')

function createRuntime(expressions, testContext) {
	var runtime = new fire.Runtime()
	runtime.moduleRequire = function(moduleName) {
		return require(moduleName)
	}
	runtime.setBaseDir(path.dirname(module.filename))
	runtime.loadModuleInstance(fireAssert, 'fire-assert')
	expressions.forEach(function(expDef){
		runtime.registerWellKnownExpressionDefinition(expDef);
	})
	runtime.load(function(initErr) {
		if(initErr) {
			testContext.callback(initErr, {
				runtime: runtime,
				error: initErr,
				result: null
			})
		}
		var expDef = runtime.getExpressionDefinition(expressions[0].name)
		var exp = new(expDef.implementation)
		exp.runtime = runtime
		exp.resultCallback = function(res) {
			testContext.callback(null, {
				runtime: runtime,
				error: null,
				result: res
			})
		};
		exp.errorCallback = function(err) {
			testContext.callback(null, {
				runtime: runtime,
				error: err,
				result: null
			})
		};
		exp.loopCallback = function() {
			throw "Loop callback catched!"
		}
		exp.run()
	})
	return runtime
}
vows.describe('fire-assert - AssertError').addBatch({
	"AssertError type must be exposed by the module": function(){
		assert.isNotNull(fireAssert.AssertError)
		assert.equal(fireAssert.AssertError, require('../lib/AssertError.js'))
	}
}).export(module)

vows.describe('fire-assert - Assert.NotEmpty').addBatch({
	"When I use @Assert.NotEmpty with no input and the last result is an empty string": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": "",
						"@Assert.NotEmpty": null
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the default message": function(err, res){
			assert.equal(res.error.error.message, "Can't be empty")
		}
	},
	"When I use @Assert.NotEmpty with an input with message and the last result is an empty string": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": "",
						"@Assert.NotEmpty": {
							"message": "Dude, it can't be empty"
						}
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the custom message": function(err, res){
			assert.equal(res.error.error.message, "Dude, it can't be empty")
		}
	},
	"When I use @Assert.NotEmpty with no input and the last result is not empty": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": "My initial and only value",
						"@Assert.NotEmpty": null
					}
				}], this)
		}
		,"there should not be errors": function(err, res){
			assert.isNull(res.error)
		}
		,"the current result should be bypassed": function(err, res){
			assert.equal(res.result, "My initial and only value")
		}
	}
}).export(module)


vows.describe('fire-assert - Assert.Empty').addBatch({
	"When I use @Assert.Empty with no input and the last result is a non-empty string": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": "Everlong",
						"@Assert.Empty": null
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the default message": function(err, res){
			assert.equal(res.error.error.message, "Must be empty")
		}
	},
	"When I use @Assert.Empty with an input with message and the last result is a non-empty string": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": "Valid value",
						"@Assert.Empty": {
							"message": "Dude, it must be empty"
						}
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the custom message": function(err, res){
			assert.equal(res.error.error.message, "Dude, it must be empty")
		}
	},
	"When I use @Assert.Empty with no input and the last result is null": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testMain",
					"json": {
						"@return": null,
						"@Assert.Empty": null
					}
				}], this)
		}
		,"there should not be errors": function(err, res){
			assert.isNull(res.error)
		}
		,"the current result should be bypassed": function(err, res){
			assert.isNull(res.result)
		}
	}
}).export(module)

vows.describe('fire-assert - Assert.Email').addBatch({
	"When I use @Assert.Email with no input and the last result is not an email": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testEmail",
					"json": {
						"@return": "some invalid email address",
						"@Assert.Email": null
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the default message": function(err, res){
			assert.equal(res.error.error.message, "Not a valid Email address")
		}
	},
	"When I use @Assert.Email with no input and the last result is a valid email": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testEmail",
					"json": {
						"@return": "chuck@norris.com",
						"@Assert.Email": null
					}
				}], this)
		}
		,"there should be no errors": function(err, res){
			assert.isNull(res.error)
		}
		,"it should bypass the result": function(err, res){
			assert.equal(res.result, "chuck@norris.com")
		}
	}
	,"When I use @Assert.Email with custom message and the last result is not an email": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "testEmail",
					"json": {
						"@return": "some invalid email address",
						"@Assert.Email": {
							"message": "come one dude, that's not an email"
						}
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the custom message": function(err, res){
			assert.equal(res.error.error.message, "come one dude, that's not an email")
		}
	}
}).export(module)
vows.describe('fire-assert - Assert.DeepEqual').addBatch({
	"When I use @Assert.DeepEqual with 'to' property equals to the last result": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "TestDeepEqual",
					"json": {
						"@return": {
							"name": "Chuck",
							"age": 12,
							"tags": ["super", "hero", "badass"]
						},
						"@Assert.DeepEqual": {
							"to": {
								"name": "Chuck",
								"age": 12,
								"tags": ["super", "hero", "badass"]
							}
						}
					}
				}], this)
		}
		,"there should be no errors": function(err, res){
			assert.isNull(res.error)
		}
		,"it should bypass the result": function(err, res){
			assert.deepEqual(res.result, {
				"name": "Chuck",
				"age": 12,
				"tags": ["super", "hero", "badass"]
			})
		}
	},
	"When I use @Assert.DeepEqual with 'to' property not equals to the last result": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "TestDeepEqual",
					"json": {
						"@return": {
							"name": "Chuck",
							"age": 12,
							"tags": ["super", "hero"]
						},
						"@Assert.DeepEqual": {
							"to": {
								"name": "Chuck",
								"age": 12,
								"tags": ["super", "hero", "badass"]
							}
						}
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the standard message": function(err, res){
			assert.equal(res.error.error.message, "Are not equal")
		}
	},
	"When I use @Assert.DeepEqual with 'to' property not equals to the last result with custom message": {
		topic: function() {
			var runtime = createRuntime([
				{
					"name": "TestDeepEqual",
					"json": {
						"@return": {
							"name": "Chuck",
							"age": 12,
							"tags": ["super", "hero"]
						},
						"@Assert.DeepEqual": {
							"to": {
								"name": "Chuck",
								"age": 12,
								"tags": ["super", "hero", "badass"]
							},
							"message": "Too bad, the obejcts are not equal"
						}
					}
				}], this)
		}
		,"the error should be an instance of AssertError": function(err, res){
			assert.isNull(res.result)
			assert.instanceOf(res.error.error, fireAssert.AssertError)
		}
		,"the error message should be the custom message": function(err, res){
			assert.equal(res.error.error.message, "Too bad, the obejcts are not equal")
		}
	}
}).export(module)